<?php
/*
schreinerwissen. auftragsverwaltung 
Copyright (C) 2011  Carsten Hilbert
c-hilbert@web.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


 /**
  * Class for creating SQL-querys
  * 
  * Can set aliases for fields
  *
  * Required: none
  * 
  * @since 2010-10
  */
class Query 
{
    protected $query_where_part = 1;
    protected $query_where_part_with_and;
    protected $query_string_order;
    protected $field_string_with_alias;
    protected $group_field;
    protected $arr_fields;
    protected $order_direction;
    protected $order_field = 'id';
    protected $table_or_relation;


    /**
     * Creates a search string on base of one or more words
     * 
     * @param array $values Werte aus dem input-feld
     * @return str $query Where part of query
     */
    function make_search_string_from_input($values) 
    {
        if (empty ($_SESSION['value1'])) {
            $query_output = 1;

        // Create an array from searching string, glue: space
        }else{
            $arr_werte = explode(" ", $values);
            $query_output = '';
            
            // Loop with one or more words
            foreach($arr_werte as $z) {
                $query_output .= "(";
                // iterate fields
                for($i=0; $i<count($this->arr_fields); $i++) { 
                    $query_output .= $this->arr_fields[$i] . " LIKE '%" . $z . "%' OR ";
                }
                $query_output = substr($query_output, 0, -4);
                $query_output .= ") AND ";
            }
            $query_output = substr($query_output, 0, -5);
        }
        $this->query_where_part = $query_output;

        return $query_output;
    }



    /**
     * @param string $param Feld nach dem sortiert wird setzen
     */
    function set_group_field($param) 
    {
        $this->group_field = $param;
    }

    /**
     * @param string $param Feld nach dem sortiert wird setzen
     */
    function set_order_field($param) 
    {
        $alias_from_order_field = $this->get_alias($param);
        $this->order_field = $alias_from_order_field;
    }

    /**
     * Sets the order direction
     * 
     * @param string $direction "ASC" or "DESC"
     */
    function set_order_direction($direction = 'ASC') 
    {
        $this->order_direction = $direction;
    }

    
    /**
     * @param string $query Where part of query
     */
    function set_query_where_part($query) 
    {
        $this->query_where_part = $query;
    }


    /**
     * @param string $query Order part of query
     */
    function set_query_string_order($query) 
    {
        $this->query_string_order = $query;
    }

    
    /**
     *
     * @param string $query Teil der Abfrage: order
     */
    function set_query_string_where_and($query) 
    {
        $this->query_where_part_with_and .= $query;
    }

    /**
     *
     * @param string $query part of query: order
     */
    function set_table_or_relation($param) 
    {
        $this->table_or_relation = $param;
    }


    /**
     * Gets field alias from 
     *  a) the language file (lang/english.inc.php)
     *  b) from array for special fields (tbl_setting.inc.php, $arr_field_alias)
     *
     * @global array $arr_fields_alias
     * @param string $field
     * @return string 
     */
    public static function get_alias($field) 
    {
        // array with special fields like auf.id, ...
        global $arr_fields_alias; 
        
        $la_var = 'la_' . $field;

        // value from language file
        if (isset($GLOBALS[$la_var])) {
            $alias = $GLOBALS[$la_var];

        // value from array in tbl_setting.php
        }elseif(isset($arr_fields_alias[$field])) {
            $alias = $arr_fields_alias[$field];

        // value like field name in all other cases
        }else{
            $alias = $field;
        }
        
        return $alias;
    }

    
    /**
     * Creates string with fields and get alias
     * f.i. id AS 'AddressID'
     * 
     * @param array $arr_fields das array aus einstellungen z.B. "id AS '$la_id'"
     * @return array $this->felder_ohne_alias
     */
    function set_fields($arr_fields) 
    {
        $this->arr_fields = $arr_fields;
        $this->make_field_string_with_alias();
    }
    
    /**
     * Makes field string with alias
     * 
     * @return type 
     */
    function make_field_string_with_alias() 
    {
        $this->field_string_with_alias = '';
       
        foreach ($this->arr_fields as $value) {
            // Get alias
            $alias = $this->get_alias($value);
            
            // Alias found
            if ($alias == true) {
                $this->field_string_with_alias .= $value . " AS '$alias',";
               
            // Alias not found
            }else{
                $this->field_string_with_alias .= " " . $value . ",";
            }
        } // end foreach
        
        $this->field_string_with_alias = substr($this->field_string_with_alias, 0, -1);
        
        return $this->field_string_with_alias;
    }


    /**
     * Gets the field string with alias
     * 
     * @return string 
     */
    function get_fields_with_alias() 
    {
        return $this->field_string_with_alias;
    }


    /**
     * Gets the sql-query
     * 
     * @return string sql-query ready to use 
     */
    function get_query() 
    {
        $qry = 'SELECT DISTINCT '. $this->field_string_with_alias;
        $qry .=  ' FROM ' . $this->table_or_relation;
        $qry .=  ' WHERE ' . $this->query_where_part;
        $qry .=  $this->query_where_part_with_and;
        if (!empty ($this->group_field)) {
            $qry .= ' GROUP BY '. $this->group_field;
        }
        if (!empty ($this->order_field)) {
            // IMPORTANT: special quotes for `alias`,'alias' don't work
            $qry .= " ORDER BY `$this->order_field` $this->order_direction ";
        }
        return $qry;
    }

    /**
     * Gets the sql-query for count the records
     * 
     * @return type 
     */
    function get_query_for_count() 
    {
        $qry = 'SELECT COUNT(*) ';
        $qry .= ' FROM '. $this->table_or_relation;
        $qry .=  ' WHERE ' . $this->query_where_part;
        $qry .=  $this->query_where_part_with_and;
        return $qry;
    }


} // ende class   
?>